#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 5
	{
		eprintln!("usage: correlate <input_a.dat> <input_b.dat> <output_forward.dat> <output_reverse.dat>");
		return;
	}

	let mut input_a = File::open(&argv[1]).unwrap();
	let mut input_b = File::open(&argv[2]).unwrap();
	let mut output_fwd = File::create(&argv[3]).unwrap();
	let mut output_rev = File::create(&argv[4]).unwrap();

	let mut data_a:Vec<f32> = Vec::new();
	let mut data_b:Vec<f32> = Vec::new();

	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_a.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data_a.push(f32::from_be_bytes(buffer));
	}

	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_b.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data_b.push(f32::from_be_bytes(buffer));
	}

	let (longer, shorter) = 
	(	
		match data_a.len() > data_b.len()
		{
			true  => (data_a, data_b),
			false => (data_b, data_a),
		}
	);

	let mut output_data_fwd:Vec<f32> = Vec::with_capacity(longer.len()+shorter.len());
	let mut output_data_rev:Vec<f32> = Vec::with_capacity(longer.len()+shorter.len());
	for offset in 0..=longer.len()+shorter.len()
	{
		let mut fwd_sum:f32 = 0.0;
		let mut rev_sum:f32 = 0.0;
		for (fi, ri) in 
		(
			0..shorter.len()
		).zip(
			(0..shorter.len()).rev()
		) {
			fwd_sum += shorter[fi] * longer.get(offset+fi-shorter.len()).unwrap_or(&0.0);
			rev_sum += shorter[ri] * longer.get(offset+fi-shorter.len()).unwrap_or(&0.0);
		}
		output_data_fwd.push(fwd_sum);
		output_data_rev.push(rev_sum);
	}

	let mut output_buffer_fwd:Vec<u8> = Vec::with_capacity(output_data_fwd.len()*4);
	for &point in output_data_fwd.iter()
	{
		output_buffer_fwd.extend_from_slice(&point.to_be_bytes());
	}

	let mut output_buffer_rev:Vec<u8> = Vec::with_capacity(output_data_rev.len()*4);
	for &point in output_data_rev.iter()
	{
		output_buffer_rev.extend_from_slice(&point.to_be_bytes());
	}

	output_fwd.write_all(&output_buffer_fwd).unwrap();
	output_rev.write_all(&output_buffer_rev).unwrap();
}
