#[allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

use std::f32::consts::PI;

fn main() 
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 4
	{
		eprintln!("usage: dcst <input.dat> <sine-output.dat> <cos-output.dat");
		return;
	}
	let mut signal_file = File::open(&argv[1]).unwrap();
	let mut iout_file = File::create(&argv[2]).unwrap();
	let mut qout_file = File::create(&argv[3]).unwrap();

	let mut signal:Vec<f32> = Vec::new();
	let mut signal_data:Vec<u8> = Vec::new();
	signal_file.read_to_end(&mut signal_data).unwrap();
	for chunk in signal_data.chunks_exact(4)
	{
		let mut samplebuffer:[u8;4] = [0x00; 4];
		samplebuffer.copy_from_slice(chunk);
		signal.push(f32::from_be_bytes(samplebuffer));
	}

	let size = signal.len();

	let mut iout:Vec<f32> = Vec::with_capacity(size);
	let mut qout:Vec<f32> = Vec::with_capacity(size);
	for n in 0..size
	{
		let mut new_i:f32 = 0.0;
		let mut new_q:f32 = 0.0;
		for t in 0..size
		{
			let (ti, tq) = (-1.0*(n as f32)*PI*(t as f32) / (size as f32)).sin_cos();
			new_i += signal[t] * ti;
			new_q += signal[t] * tq;
		}
		new_i /= (size as f32).sqrt();
		new_q /= (size as f32).sqrt();
		iout.push(new_i);
		qout.push(new_q);
	}

	let mut iout_data:Vec<u8> = Vec::with_capacity(size*4);
	let mut qout_data:Vec<u8> = Vec::with_capacity(size*4);
	for n in 0..size
	{
		iout_data.extend_from_slice(&iout[n].to_be_bytes());
		qout_data.extend_from_slice(&qout[n].to_be_bytes());
	}

	iout_file.write_all(&iout_data).unwrap();
	qout_file.write_all(&qout_data).unwrap();
}
