#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

use std::f32::consts::PI;

fn gauss
(
	freq: f32,
	input: f32,
) ->
	f32
{
	let stdev = (2.0 * PI * freq).recip();
	(
		-0.5 *
		(
			input
			/
			stdev
		)
		.powi(2)
	)
	.exp()
	/
	(
		(2.0 * PI).sqrt()
		*
		stdev
	)
}

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 3
	{
		eprintln!("usage: gfilter <frequency> <target.dat>");
		return;
	}

	let filter_frequency = 
	(
		argv[1]
		.parse::<f32>()
		.unwrap()
		/ 2000.0
	);

	let mut input_file = File::open(&argv[2]).unwrap();
	let mut data:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_file.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data.push(f32::from_be_bytes(buffer));
	}

	let mut filtered_data:Vec<f32> = Vec::with_capacity(data.len());
	for shift in 0..data.len()
	{
		let mut data_sum:f32 = 0.0;
		for x in 0..data.len()
		{
			data_sum += 
			(
				data[x] 
				* 
				gauss(
					filter_frequency,
					(x as f32) - (shift as f32)
				)
			);
		}
		filtered_data.push(data_sum);
	}

	let mut output_file = File::create(&argv[2]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(filtered_data.len() * 4);
	for &point in filtered_data.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output_file.write_all(&output_buffer).unwrap();
}
