#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;
use std::process::exit;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 2
	{
		eprintln!("no. arguments: <target.dat>");
		return;
	}

	let mut input = File::open(&argv[1]).unwrap();
	let mut data:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data.push(f32::from_be_bytes(buffer));
	}

	let mut abs_max:f32 = 0.0;
	for &point in data.iter()
	{
		if point.is_normal()
		{
			abs_max = abs_max.max(point.abs());
		}
	}

	if 
	(
		abs_max < std::f32::MIN_POSITIVE
	) {
		eprintln!("cannot normalize {}! it may be empty, it may contain one or more infinite or NaN values, or it may contain only values that are zero or nearly zero.", argv[1]);
		exit(1);
	}

	for point in data.iter_mut()
	{
		*point /= abs_max;
	}

	let mut output = File::create(&argv[1]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(data.len() * 4);
	for &point in data.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output.write_all(&output_buffer).unwrap();
}
