#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 2
	{
		eprintln!("usage: peakdetect <target.dat>");
		return;
	}

	let mut input_file = File::open(&argv[1]).unwrap();
	let mut data:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_file.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data.push(f32::from_be_bytes(buffer));
	}

	let mut filtered_data:Vec<f32> = Vec::with_capacity(data.len());
	filtered_data.push(0.0);
	for i in 1..data.len()-1
	{
		let mut upper_radius:usize = 1;
		let mut lower_radius:usize = 1;
		while i.saturating_add(upper_radius) < (data.len()-1) && data[i] == data[i+upper_radius]
		{
			upper_radius += 1;
		}
		while i.saturating_sub(lower_radius) > 0 && data[i] == data[i-lower_radius]
		{
			lower_radius += 1;
		}
		if 
		(
			(data[i-lower_radius].abs() < data[i].abs() && data[i+upper_radius].abs() < data[i].abs())
		) {
			filtered_data.push(data[i]);
		} else {
			filtered_data.push(0.0);
		}
	}
	filtered_data.push(0.0);

	let mut output_file = File::create(&argv[1]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(filtered_data.len() * 4);
	for &point in filtered_data.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output_file.write_all(&output_buffer).unwrap();
}
