#[allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

use std::f32::consts::PI;

fn main() 
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 5
	{
		eprintln!("usage: sft <i-input.dat> <q-input.dat> <i-output.dat> <q-output.dat>");
		return;
	}

	let mut iout_file = File::create(&argv[3]).unwrap();
	let mut qout_file = File::create(&argv[4]).unwrap();

	let mut isig_file = File::open(&argv[1]).unwrap();
	let mut isig_data:Vec<u8> = Vec::new();
	isig_file.read_to_end(&mut isig_data).unwrap();
	let mut isig:Vec<f32> = Vec::new();
	for chunk in isig_data.chunks_exact(4)
	{
		let mut samplebuffer:[u8;4] = [0x00; 4];
		samplebuffer.copy_from_slice(chunk);
		isig.push(f32::from_be_bytes(samplebuffer));
	}

	let mut qsig_file = File::open(&argv[2]).unwrap();
	let mut qsig_data:Vec<u8> = Vec::new();
	qsig_file.read_to_end(&mut qsig_data).unwrap();
	let mut qsig:Vec<f32> = Vec::new();
	for chunk in qsig_data.chunks_exact(4)
	{
		let mut samplebuffer:[u8;4] = [0x00; 4];
		samplebuffer.copy_from_slice(chunk);
		qsig.push(f32::from_be_bytes(samplebuffer));
	}

	let size = isig.len().min(qsig.len());

	let mut iout:Vec<f32> = Vec::with_capacity(size);
	let mut qout:Vec<f32> = Vec::with_capacity(size);
	for n in 0..size
	{
		let mut new_i:f32 = 0.0;
		let mut new_q:f32 = 0.0;
		for t in 0..size
		{
			// (R+Bi)*(C+Di) = (AC-BD)+(AD+BC)i
			let (tq, ti) = (-2.0*(n as f32)*PI*(t as f32) / (size as f32)).sin_cos();
			new_i += isig[t]*ti - qsig[t]*tq;
			new_q += isig[t]*tq + qsig[t]*ti;
		}
		new_i /= (size as f32).sqrt();
		new_q /= (size as f32).sqrt();
		iout.push(new_i);
		qout.push(new_q);
	}

	let mut idata:Vec<u8> = Vec::with_capacity(size*4);
	let mut qdata:Vec<u8> = Vec::with_capacity(size*4);
	for n in 0..size
	{
		idata.extend_from_slice(&iout[n].to_be_bytes());
		qdata.extend_from_slice(&qout[n].to_be_bytes());
	}

	iout_file.write_all(&idata).unwrap();
	qout_file.write_all(&qdata).unwrap();
}
