#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

use std::f32::consts::PI;

fn sinc
(
	x: f32,
) ->
	f32
{
	if x.abs() > std::f32::MIN_POSITIVE
	{
		(PI*x).sin() / (PI*x)
	} else {
		1.0
	}
}

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 4
	{
		eprintln!("usage: swindow <low> <high> <target.dat>");
		return;
	}

	let freq_lo = 
	(
		argv[1]
		.parse::<f32>()
		.unwrap()
		/ 2000.0
	);
	let freq_hi = 
	(
		argv[2]
		.parse::<f32>()
		.unwrap()
		/ 2000.0
	);

	let mut input_file = File::open(&argv[3]).unwrap();
	let mut input_buffer:Vec<u8> = Vec::new();
	input_file.read_to_end(&mut input_buffer).unwrap();
	let mut data:Vec<f32> = Vec::new();
	for chunk in input_buffer.chunks_exact(4)
	{
		let mut new_value:[u8;4] = [0x00;4];
		new_value.copy_from_slice(chunk);
		data.push(f32::from_be_bytes(new_value));
	}

	let mut filtered_data:Vec<f32> = Vec::with_capacity(data.len());
	for shift in 0..data.len()
	{
		let s = shift as f32;

		let mut value_hi:f32 = 0.0;
		let mut value_lo:f32 = 0.0;
		for x in 0..data.len()
		{
			let t = x as f32;

			value_hi += 
			(
				data[x] 
				* 
				(2.0 * freq_hi)
				*
				sinc(
					(2.0 * freq_hi)
					*
					(t - s)
				)
			);

			value_lo +=
			(
				data[x] 
				* 
				(2.0 * freq_lo)
				*
				sinc(
					(2.0 * freq_lo)
					*
					(t - s)
				)
			);

			filtered_data.push(value_hi - value_lo);
		}
	}

	let mut output_file = File::create(&argv[3]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(filtered_data.len() * 4);
	for &point in filtered_data.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output_file.write_all(&output_buffer).unwrap();
}
