use std::io::prelude::*;
use std::fs::File;
use std::env::args;

const TAU:f32 = std::f32::consts::PI*2.0;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 5
	{
		eprintln!("no. <length> <start> <stop> <output.dat>");
		return;
	}
	let length = argv[1].parse::<usize>().unwrap();
	let start_freq = 1.0 / argv[2].parse::<f32>().unwrap();
	let stop_freq = 1.0 / argv[3].parse::<f32>().unwrap();
	let mut output_file = File::create(&argv[4]).unwrap();

	let delta_freq = (stop_freq-start_freq) / (length as f32);
	let mut output_buffer:Vec<u8> = Vec::with_capacity(length * 4);
	for i in 0..length
	{
		output_buffer.extend_from_slice(
			&(
				(i as f32) * TAU 
				/
				(start_freq + delta_freq * (i as f32))
				/
				(length as f32)
			)
			.sin()
			.to_be_bytes()
		)
	}
	output_file.write_all(&output_buffer).unwrap();
}
