#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

use std::f32::consts::PI;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 4
	{
		eprintln!("usage: unitvex <input.dat> <output_i.dat> <output_q.dat>");
		return;
	}

	let mut infile = File::open(&argv[1]).unwrap();
	let mut indata:Vec<u8> = Vec::new();
	infile.read_to_end(&mut indata).unwrap();

	let mut input:Vec<f32> = Vec::with_capacity(indata.len()/4);
	for chunk in indata.chunks_exact(4)
	{
		let mut samplebuffer:[u8;4] = [0x00; 4];
		samplebuffer.copy_from_slice(chunk);
		input.push(f32::from_be_bytes(samplebuffer));
	}

	let mut idata:Vec<u8> = Vec::with_capacity(indata.len());
	let mut qdata:Vec<u8> = Vec::with_capacity(indata.len());
	for point in input.iter()
	{
		let new_i = (2.0*PI*point).cos();
		let new_q = (2.0*PI*point).sin();
		idata.append(&mut new_i.to_be_bytes().to_vec());
		qdata.append(&mut new_q.to_be_bytes().to_vec());
	}

	let mut outfile_i = File::create(&argv[2]).unwrap();
	let mut outfile_q = File::create(&argv[3]).unwrap();
	outfile_i.write_all(&idata).unwrap();
	outfile_q.write_all(&qdata).unwrap();
}
