#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 4
	{
		eprintln!("usage: vecsum <input_a.dat> <input_b.dat> <output.dat>");
		return;
	}

	let mut input_a = File::open(&argv[1]).unwrap();
	let mut data_a:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_a.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data_a.push(f32::from_be_bytes(buffer));
	}
	let mut input_b = File::open(&argv[2]).unwrap();
	let mut data_b:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_b.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data_b.push(f32::from_be_bytes(buffer));
	}

	let mixed_len = 
	(
		data_a.len()
	).max(
		data_b.len()
	);

	let mut data_mixed:Vec<f32> = Vec::with_capacity(mixed_len);
	for i in 0..mixed_len
	{
		data_mixed.push(
			(
				(*data_a.get(i).unwrap_or(&0.0)).powi(2)
				+
				(*data_b.get(i).unwrap_or(&0.0)).powi(2)
			).sqrt()
		);
	}

	let mut output_file = File::create(&argv[3]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(mixed_len * 4);
	for &point in data_mixed.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output_file.write_all(&output_buffer).unwrap();
}
