#![allow(unused_parens)]

use std::io::prelude::*;
use std::fs::File;
use std::env::args;

const RAND_DEVICE:&str = "/dev/urandom";

fn main()
{
	let argv = args().collect::<Vec<String>>();
	if argv.len() != 3
	{
		eprintln!("usage: worsen <SNR‰> <target.dat>");
		return;
	}

	let snr = 
	(
		(
			argv[1]
			.parse::<f32>()
			.unwrap()
			/ 1000.0
		)
		.max(0.0)
		.min(1.0)
	);

	let mut input_signal = File::open(&argv[2]).unwrap();
	let mut data_signal:Vec<f32> = Vec::new();
	loop
	{
		let mut buffer:[u8;4] = [0x00;4];
		match input_signal.read_exact(&mut buffer)
		{
			Ok(_) => (),
			Err(_) => break,
		};
		data_signal.push(f32::from_be_bytes(buffer));
	}
	let mut input_noise = File::open(&RAND_DEVICE).unwrap();
	let mut data_noise:Vec<f32> = Vec::new();
	for _ in 0..data_signal.len()
	{
		let mut buffer:[u8;4] = [0x00;4];
		input_noise.read_exact(&mut buffer).unwrap();
		data_noise.push(
			(
				(i32::from_be_bytes(buffer) as f64)
				/ 
				(i32::max_value() as f64)
			)
			as f32
		);
	}

	let signal_level = snr;
	let noise_level = 1.0 - snr;

	let mut data_mixed:Vec<f32> = Vec::with_capacity(data_signal.len());
	for i in 0..data_signal.len()
	{
		data_mixed.push(
			signal_level * data_signal[i]
			+
			noise_level * data_noise[i]
		);
	}

	let mut output_file = File::create(&argv[2]).unwrap();
	let mut output_buffer:Vec<u8> = Vec::with_capacity(data_signal.len() * 4);
	for &point in data_mixed.iter()
	{
		output_buffer.extend_from_slice(&point.to_be_bytes());
	}
	output_file.write_all(&output_buffer).unwrap();
}
